from functools import wraps
import logging
import os
from pscr_common.log import setup_lambda_logging

from org_retention.utils.config import Config

_logger = logging.getLogger(__name__)


def setup_lambda_handler(config: Config):
    def decorator(f):
        @wraps(f)
        def wrapper(event, context):
            try:
                setup_lambda_logging(
                    level=config.get("log", "level", fallback="INFO"),
                    svc_version=config.get("service", "version"),
                    env_id=os.environ.get("ENV_ID", "UNKNOWN"),
                    svc_id=context.aws_request_id,
                )

                _logger.debug("Event: %s", event)

                if event.get("route") == "version":
                    return {"version": config.get("service", "version")}

                return f(event, context)
            except Exception:
                _logger.exception(
                    "Error during lambda handler (event: %s context: %s)",
                    event,
                    context,
                )
                raise

        return wrapper

    return decorator
