import asyncio
import logging

_logger = logging.getLogger(__name__)


class BackgroundTask(object):
    def __init__(self, name):
        self.name = name
        self.alive = True

    async def context(self, app) -> None:
        app[self.name] = asyncio.get_event_loop().create_task(self.run(app))

        yield

        app[self.name].cancel()
        await app[self.name]

    def register(self, app, instance_number=None) -> None:
        if instance_number is not None:
            self.name = f"{self.name}-{instance_number}"
        app.cleanup_ctx.append(self.context)

    async def run(self, app) -> None:
        while self.alive:
            try:
                await self.execute(app)
            except asyncio.CancelledError:
                _logger.info("Task '%s' cancelled", self.name)
                self.alive = False
            except:
                _logger.exception(f"Unknown error during {self.name}")

    async def execute(self, app) -> None:
        raise NotImplementedError
