import logging

import rapidjson
from aiohttp import web

from org_retention.engine.retention_engine import RetentionEngine
from pscr_common.web.auth import AuthRouteTableDef

from org_retention.utils import json_dumps

_logger = logging.getLogger(__name__)


def make_response_json(data=None) -> web.Response:
    if data is None:
        return web.Response(status=204)
    else:
        return web.json_response(data, dumps=json_dumps)


def build_route_table(retention_engine: RetentionEngine) -> AuthRouteTableDef:
    routes = AuthRouteTableDef()

    @routes.get("/v1/orgs")
    async def get_all_retentions(_request: web.Request):
        return make_response_json(
            {"retentions": await retention_engine.get_all_retentions()}
        )

    @routes.get("/v1/defaults")
    async def get_retention_defaults(_request: web.Request):
        return make_response_json(await retention_engine.get_retention_class_defaults())

    @routes.get("/v1/allowed_days")
    async def get_allowed_days(_request: web.Request):
        return make_response_json(
            await retention_engine.get_retention_class_allowed_days()
        )

    @routes.post("/v1/orgs/schedule_update_retention_config")
    async def schedule_update_retention_config(_request: web.Request):
        retention_engine.schedule_update_retention_config()
        return web.Response(status=202)

    return routes
